local signal = require('lib.signal')
local audio = require('utils.audio')
local vec = require('lib.vector-light')
local colors = require('utils.colors')
local filters = require('utils.filters')
local explosion_particles = require('particles.explosion_particles')
local cam = require('cam')
local game = nil
local Explosion = {}
local mt = {
  is_explosion = true
}
mt.__index = mt

local RADIUS = 60

function mt:getY()
  return self.y + 1000 -- above everything else
end

function mt:getRect()
  return self.x-5, self.y-5, 10, 10
end

function mt:update(dt)
  self.t = self.t + dt

  if self.psystem then
    self.psystem:update(dt)
  else
    if self.r < 32 then
      self.r = self.r + 250 * dt
    else
      self.color = colors.black
    end

    if self.t > 0.3 then
      self.psystem = explosion_particles.new()
      self.psystem:moveTo(self.x, self.y)
      self.psystem:emit(10)
    end
  end

  if self.t > 0.6 then
    game.remove(self)
  end
end

function mt:draw()
  if self.psystem then
    love.graphics.setColor(colors.white)
    love.graphics.draw(self.psystem)
  else
    love.graphics.setColor(self.color)
    love.graphics.circle('fill', self.x, self.y, self.r, RADIUS)
    love.graphics.setColor(colors.white)
  end
end

function Explosion.setup(_game)
  game = _game
end

function Explosion.new(x, y, spawner)
  audio.playRandom('explosion')

  signal.emit('explosion', x, y) -- map handles tiles and other players? probably should be here

  local items, len = game.world:queryRect(x-RADIUS/2, y-RADIUS/2, RADIUS, RADIUS, filters.queryBombOrBox)

  for i, item in ipairs(items) do
    local _x, _y = item:getOrigin()
    if item ~= spawner and vec.len(_x-x, _y-y) <= RADIUS then
      item:ignite()
    end
  end

  return setmetatable({
    x = x, y = y, r = 5, t = 0, color = colors.white
  }, mt)
end

return Explosion
