local assets = require('utils.assets')
local audio = require('utils.audio')
local colors = require('utils.colors')
local dust = require('particles.tile_dust')
local signal = require('lib.signal')

local SHAKE_MAX_T = 1
local FALL_GRAVITY = 2000 -- more then player's so it falls faster
local Tile = {}
local tex = nil
local tilesets = {}

local mt = {
  is_tile = true
}
mt.__index = mt

function mt:draw()
  love.graphics.setColor(255, 255, 255)
  local rx, ry = 0, 0
  if self.intro_y > 0.01 then
    rx = love.math.random(-1, 1)
  end
  local r = 0
  if self.shake_t > 0 then
    local sp = math.min(self.shake_power, 0.2)
    local max = 0.6 * self.shake_t/SHAKE_MAX_T
    r = love.math.random() * max*sp - max*sp*0.5
  end

  if self.till_fall_t then
    r = love.math.random() * 0.3 - 0.15
  end

  local _x, _y = self.x+rx+8, self.y+8+self:getYOffset()

  local nalpha = (math.max(0, self.hp) / 3)^2
  local c = math.max(0, math.min(180 + nalpha * 75, 255))
  love.graphics.setColor(c, c, c)
  love.graphics.draw(tex, self.q, _x, _y, r, 1, 1, 8, 8)

  if self.color then
    love.graphics.setColor(self.color)
    assets.draw('symbol', _x+6, _y+6, r, 1, 1, 8, 8)
  end


  love.graphics.setColor(255, 255, 255)
  -- love.graphics.print(("%d\n%.1f"):format(self.hp, nalpha), _x-6, _y-5, 0, 0.333, 0.333)
end

function mt:getCenter()
  return self.x+17/2, self.y+17/2
end

function mt:lateDraw()
  love.graphics.setColor(255, 255, 255)
  love.graphics.draw(self.dust)
end

function mt:getYOffset()
  return self.intro_y + self.fall_z
end

function mt:update(dt)
  self.shake_t = self.shake_t - dt
  self.dust:update(dt)
  self.delay = self.delay - dt

  if self.intro_y < 0.4 and not self.emit_flag then
    self.emit_flag = true
    self.dust:setPosition(self.x+8, self.y+8)
    self.dust:start() self.dust:emit(3) self.dust:stop()
    signal.emit('tile_intro')
    audio.playRandom('tile_up', 1.0)
  end

  if self.is_falling then
    self.fall_vz = (self.fall_vz or 0) + FALL_GRAVITY * dt
    self.fall_z = self.fall_z + self.fall_vz * dt
  end

  if self.delay < 0 then
    self.intro_y = (self.intro_y > 0) and (self.intro_y / 3) or 0
  end

  if self.till_fall_t then
    self.till_fall_t = self.till_fall_t - dt

    if self.till_fall_t < -0.1 then
      game.remove(self)
    end

    if self.till_fall_t < 0 then
      -- start falling

      if not self.is_falling then
        audio.playRandom('tile_fall')
      end

      self.is_falling = true
      self.is_tile = false
    end
  else
    if self.hp <= 0 then
      self.till_fall_t = 0.75 + love.math.random() * 0.5
      signal.emit('recalc_points')
    end
  end
end

function mt:getRect()
  return self.x, self.y, 17, 17
end

function mt:getY()
  return self.y
end

function mt:damage(val)
  self.hp = self.hp - val
end

function mt:shake(pow, t)
  self.shake_power = pow
  self.shake_t = t
end

function Tile.setup()
  tex = assets.img('tex.png')
  tilesets['basic'] = assets.quadsGroup('tiles_basic')
end

function Tile.new(qid, x, y, _delay, color)
  local t = setmetatable({
    TMP = love.math.random(1,20),
    dust = dust.new(),
    intro_y = 1000,
    delay = _delay or 0,
    x = x, y = y,
    q = tilesets['basic'][qid],
    shake_t = 0,
    hp = love.math.random() * 2 + 2,
    fall_z = 0,
  }, mt)

  if color then
    t.color_name = color
    t.player_id = colors.nameToId(color)
    local c = colors[color]
    t.color = { c[1], c[2], c[3], 90 }
  end

  return t
end

return Tile
