local signal = require('lib.signal')
local vec = require('lib.vector-light')

local DEAD_ZONE = 0.2

local function _p()
  local _s = function() return { count=0, time=0 } end
  return { jump=_s(), throw=_s(), menu=_s(), cancel=_s(), down=_s(), up=_s(), right=_s(), left=_s(),
    confirm1=_s(),
    confirm2=_s()
  }
end

local input = {}

local _assign_requests = {}
local _devide2id = {}
local _id2device = {}
local _state = { _p(), _p(), _p(), _p() }

local function _isHold(player_id, action_name) return _state[player_id][action_name].count > 0 end
local function _isHit(player_id, action_name) return _state[player_id][action_name].count == 1 end

local function _makeAxesValSafe(n)
  local absn = math.abs(n)
  if absn < DEAD_ZONE then return 0 end

  local sign = (n > 0 and 1 or -1)

  return ((absn - DEAD_ZONE) / (1 - DEAD_ZONE)) * sign -- (0,1)
end

function input.getHoldT(player_id, action_name) return _state[player_id][action_name].time end

function input.isJumpHold(player_id) return _isHold(player_id, 'jump') end
function input.isJumpHit(player_id) return _isHit(player_id, 'jump') end

function input.isConfirmHit(player_id)
  return
    input.isJumpHit(player_id) or
    _isHit(player_id, 'confirm1') or -- alias space?
    _isHit(player_id, 'confirm2') -- alias enter
end
function input.isMenuHit(player_id) return _isHit(player_id, 'menu') end
function input.isCancelHit(player_id) return _isHit(player_id, 'cancel') end

function input.isThrowHold(player_id) return _isHold(player_id, 'throw') end
function input.isThrowHit(player_id) return _isHit(player_id, 'throw') end

function input.isUpHit(player_id) return _isHit(player_id, 'up') end
function input.isDownHit(player_id) return _isHit(player_id, 'down') end
function input.isLeftHit(player_id) return _isHit(player_id, 'left') end
function input.isRightHit(player_id) return _isHit(player_id, 'right') end

function input.anyHit(player_id)
  return _isHit(player_id, 'throw') or
         _isHit(player_id, 'menu') or
         _isHit(player_id, 'jump') or
         _isHit(player_id, 'cancel') or
         input.isConfirmHit(player_id)
end

function input.deviceToPlayerId(device)
  return _devide2id[device]
end

function input.getAxes(player_id)
  local device = _id2device[player_id]
  -- if device == 'kb' then
  --   local d = love.keyboard.isDown
  --   return vec.normalize(
  --
  --     (d('left') and -1 or 0) + (d('right') and 1 or 0),
  --     (d('up') and -1 or 0) + (d('down') and 1 or 0)
  --   )

  if type(device) == 'string' then -- 'kb' or 'bot_[n]'
    return vec.normalize(
      (_isHold(player_id, 'left') and -1 or 0) + (_isHold(player_id, 'right') and 1 or 0),
      (_isHold(player_id, 'up') and -1 or 0) + (_isHold(player_id, 'down') and 1 or 0)
    )

  elseif device then
    return _makeAxesValSafe(device:getGamepadAxis('leftx')),
          _makeAxesValSafe(device:getGamepadAxis('lefty'))

  else
    return 0, 0
  end
end
function input.getAllAxisHit()
end

function input.unassign(player_id)
  local d = _id2device[player_id]
  _id2device[player_id] = nil
  if d then
    _devide2id[d] = nil
  end
end

function input.setAssignmentReq(id, val)
  _assign_requests[id] = val
end

function input.set(id, action_name, hold)
  _state[id][action_name].hold = hold
end

function input.assign(player_id, device)
  -- device == 'kb' or a Joystick

  if player_id == 'next' then
    if not _id2device[1] then player_id = 1 end
    if not _id2device[2] then player_id = 2 end
    if not _id2device[3] then player_id = 3 end
    if not _id2device[4] then player_id = 4 end
  end

  _devide2id[device] = player_id
  _id2device[player_id] = device
end

signal.register('keypressed', function(k)
  if k == 'x' or k == 'c' then
    _assign_requests['kb'] = true
  end

  local id = _devide2id['kb']
  if not id then return end

  if k == 'up' then _state[id].up.hold = true
  elseif k == 'down' then _state[id].down.hold = true
  elseif k == 'left' then _state[id].left.hold = true
  elseif k == 'right' then _state[id].right.hold = true
  elseif k == 'x' then _state[id].jump.hold = true
  elseif k == 'c' then _state[id].throw.hold = true
  elseif k == 'space' then _state[id].confirm1.hold = true
  elseif k == 'return' then _state[id].confirm2.hold = true
  elseif k == 'escape' then _state[id].menu.hold = true
  end
end)

signal.register('keyreleased', function(k)
  if k == 'x' or k == 'c' then
    _assign_requests['kb'] = nil
  end

  local id = _devide2id['kb']
  if not id then return end

  if k == 'up' then _state[id].up.hold = false
  elseif k == 'down' then _state[id].down.hold = false
  elseif k == 'left' then _state[id].left.hold = false
  elseif k == 'right' then _state[id].right.hold = false
  elseif k == 'x' then _state[id].jump.hold = false
  elseif k == 'c' then _state[id].throw.hold = false
  elseif k == 'space' then _state[id].confirm1.hold = false
  elseif k == 'return' then _state[id].confirm2.hold = false
  elseif k == 'escape' then _state[id].menu.hold = false
  end
end)

signal.register('gamepadpressed', function(j, b)
  if b == 'a' or b == 'b' then
    _assign_requests[j] = true
  end

  local id = _devide2id[j]
  if not id then return end

  if b == 'a' then _state[id].jump.hold = true
  elseif b == 'x' then _state[id].throw.hold = true
  elseif b == 'start' then _state[id].menu.hold = true
  elseif b == 'b' then _state[id].cancel.hold = true
  end
end)

signal.register('gamepadreleased', function(j, b)
  if b == 'a' or b == 'b' then
    _assign_requests[j] = nil
  end

  local id = _devide2id[j]
  if not id then return end

  if b == 'a' then _state[id].jump.hold = false
  elseif b == 'x' then _state[id].throw.hold = false
  elseif b == 'start' then _state[id].menu.hold = false
  elseif b == 'b' then _state[id].cancel.hold = false
  end
end)

signal.register('gamepadaxis', function(j, axis, val)
  local id = _devide2id[j]
  if not id then return end

  if axis == 'lefty' then _state[id].down.hold, _state[id].up.hold = (val > 0.5), (val < -0.5) end
  if axis == 'leftx' then _state[id].right.hold, _state[id].left.hold = (val > 0.5), (val < -0.5) end
end)

function _ua(state, action_name, dt)
  local a = state[action_name]
  a.count = a.hold and (a.count + 1) or 0
  a.time = a.hold and (a.time + dt) or 0
end

function input.update(dt)
  for i, v in ipairs(_state) do
    _ua(v, 'jump', dt)
    _ua(v, 'throw', dt)
    _ua(v, 'up', dt)
    _ua(v, 'down', dt)
    _ua(v, 'left', dt)
    _ua(v, 'right', dt)
    _ua(v, 'menu', dt)
    _ua(v, 'cancel', dt)
    _ua(v, 'confirm1', dt)
    _ua(v, 'confirm2', dt)
  end
end

function input.resetAssignRequests()
  _assign_requests = {} -- just in case
end

function input.iterAssignRequests()
  return pairs(_assign_requests)
end

function input.clearAssignments()
  input.resetAssignRequests()
  for id=1,4 do input.unassign(id) end
end

function input.setup()
  local joysticks = love.joystick.getJoysticks()
  input.assign(1, joysticks[1] or 'kb')
  input.unassign(2)
  input.unassign(3)
  input.unassign(4)
end

return input
