--[[

local PlayerStates = fsm.define('idle', {
  running = { idle = true, jump_rise = true }
  jump_rise = { jump_pick = true }
  jump_peak = {
    RULES = { go_to = 'jump_fall', after = 0.2 }
    jump_fall = true
  }
  idle = {
    RULES = { for_at_least = 0.1 }

  }
})

state = PlayerStates.new()

state:is('idle')
state:arrange('running')
state:update(0.016667)
state:is('running')
state:was('idle')

]]--

local fsm = {
  debug = true
}

local mt = {}; mt.__index = mt

function mt:_err(str)
  if fsm.debug then
    error(str)
  end
end

function mt:_isFrozen()
  return curr.RULES and curr.RULES.for_at_least and curr.RULES.for_at_least < self.t
end

-- sets the state following rules of transitions in next update
function mt:arrange(name)
  local curr = self.def[self.state]
  if curr[name] then
    if not self:_isFrozen() then
      self.suggested_state = name
    end
  else
    self:_err('invalid transtion: ' .. tostring(curr) .. ' -> ' .. tostring(name))
  end
end

-- sets the state ignoring the rules of transtion (to be avoided)
function mt:force(name)
  self.state = name
  self.suggested_state = name
end

-- returns current state
function mt:get()
  return self.state
end

-- checks if it was `name` state the frame before current one
function mt:was(name)
  return self.prev_frame_state == name
end

-- checks if current state is equal `name`, same as :get() == name
function mt:is(name)
  return self.state == name
end

function mt:update(dt)
  self.prev_frame_state = self.state
  self.t = self.t + dt

  if self.suggested_state ~= self.state then
    self.state = self.suggested_state
    self.t = 0
  end
end

function fsm.define(init_state, def, debug)
  return {
    new = function()
      return setmetatable({
        def = def,
        t=0,
        prev_frame_state = init_state,
        state = init_state,
        suggested_state = init_state,
        debug=(not not debug)
      }, mt)
    end
  }
end

return fsm
