local _OTHER_VAL = {} -- ~= to everything else
local ism = {}

local mt = {}
mt.__index = mt

local _ncache = {
  Update = {}, Start = {}, End = {}
}

local function _upper(s) return s:gsub('_', ''):upper() end
local function _buildFnName(s, suffix)
  -- it's rather primitive and limited but if you only have foo_bar format it'll work
  local name = 'on' .. s:gsub('^.', _upper):gsub('_.', _upper) .. suffix
  _ncache[suffix][s] = name
  return name
end

function mt:update(dt)
  local actor = self.actor

  if self.suggestion ~= self.state then
    if self.state ~= _OTHER_VAL then
      local endFn = _ncache.End[self.state] or _buildFnName(self.state, 'End')
      if actor[endFn] then actor[endFn](actor) end
    end

    local startFn = _ncache.Start[self.suggestion] or _buildFnName(self.suggestion, 'Start')
    if actor[startFn] then actor[startFn](actor) end

    self.state = self.suggestion
    self.t = 0
  end

  local updateFn = _ncache.Update[self.state] or _buildFnName(self.state, 'Update')
  if actor[updateFn] then actor[updateFn](actor, dt) end

  self.t = self.t + dt
end

function mt:set(state)
  self.suggestion = state
end

function mt:getT()
  return self.t
end

function mt:getState()
  return self.state
end

function mt:is(state_name)
  return self.state == state_name
end

function mt:after(t, state)
  if t < self.t then
    if type(state) == 'string' then self:set(state)
    elseif type(state) == 'function' then state(self.actor) -- will be called continuously
    end
  end
end

function ism.new(actor, init_state)
  return setmetatable({
    t = 0,
    actor = actor,
    state = _OTHER_VAL,
    suggestion = init_state,
  }, mt)
end

return ism

-- self.ism = ism.new(self, 'idle')
--
-- self.ism:update(dt) -- calls ~onIdleStart~, onIdleUpdate
-- self.ism:update(dt) -- calls onIdleUpdate
--
-- self.ism:set('jump') -- sets internal state, but offects nothing for now
--
-- self.ism:set('run') -- last :set wins
--
-- self.ism:update(dt) -- calls onIdleEnd, onRunStart, onRunUpdate
--
--
-- function mt:onThrowStart()
--   audio.playRandom('throw')
-- end
-- function mt:onThrowUpdate(dt)
--   -- self:onCommonUpdate(dt) -- ha!
--   self.ism:after(0.3, 'idle')
-- end
