game = require('game')
local input = require('input')
local options = require('options')

local signal = require('lib.signal')

function love.load()
  love.graphics.setDefaultFilter('nearest', 'nearest')
  options.load()

  input.setup()
  game.setup()

  require('entities.player').setup(game)
  require('entities.tile').setup(game)
  require('entities.box').setup(game)
  require('entities.bomb').setup(game)
  require('entities.explosion').setup(game)

  game.setState('menu')
end

function love.update(dt)
  game.update(dt)
end

local _init_time = love.timer.getTime()
function love.draw()
  game.draw()

  if game.debug_info then
    love.graphics.origin()
    love.graphics.setColor(255, 255, 255)
    local stats = love.graphics.getStats()
    love.graphics.print(
           'FPS:        ' .. love.timer.getFPS()
      .. '\nMEM:        ' .. ('%.3f MB'):format(collectgarbage('count') / 1024)
      .. '\n'
      .. '\ndrawcalls:  ' .. stats.drawcalls
      .. '\nc switches: ' .. stats.canvasswitches
      .. '\ntex mem:    ' .. ('%.3f MB'):format(stats.texturememory / 1024 / 1024)
      .. '\n'
      .. '\nT:          ' .. ('%.2f s'):format(love.timer.getTime() - _init_time)
    , 5, 5)
  end
end

function love.textinput(t)
  signal.emit('textinput', t)
end

function love.keypressed(k)
  signal.emit('keypressed', k)
  game.keypressed(k)
end

function love.keyreleased(k)
  signal.emit('keyreleased', k)
end

function love.gamepadpressed(j, btn)
  signal.emit('gamepadpressed', j, btn)
end

function love.gamepadreleased(j, btn)
  signal.emit('gamepadreleased', j, btn)
end

function love.gamepadaxis(j, axis, val)
  signal.emit('gamepadaxis', j, axis, val)
end

function love.joystickadded(j)
  -- todo handle
end

function love.joystickremoved(j)
  -- todo handle
end
