local later = require('utils.later')
local Tile = require('entities.tile')
local Bomb = require('entities.bomb')
local game = nil
local map = nil

local BasicMap = {}

function BasicMap.getPlayerDelay(id)
  return 1.5 + id * 0.4
end

function BasicMap.getPlayersReleaseTime()
  return 4.5
end

function BasicMap.getPlayerDir(id)
  -- also could depend on number of players!
  if id == 1 or id == 4 then
    return 1, 0
  else
    return -1, 0
  end
end

function _getTileColor(x, y)
  if map.numberOfPlayers == 4 then
    if x <= 6 and y <= 4 then return 'red' end
    if x > 14 and y <= 4 then return 'blue' end
    if x > 14 and y > 6 then return 'orange' end
    if x <= 6 and y > 6 then return 'green' end

  elseif map.numberOfPlayers == 3 then
    if x <= 6 and y <= 4 then return 'red' end
    if x > 14 and y <= 4 then return 'blue' end
    if (8 <= x and x <= 13) and y > 6 then return 'orange' end

  elseif map.numberOfPlayers == 2 then
    if x <= 6 and y <= 4 then return 'red' end
    if x > 14 and y > 6 then return 'blue' end
  end
end

function BasicMap.getInitBoxPosition(pos)
  if pos == 1 then return 134, 51+17*2 end
  if pos == 2 then return 278, 51+17*2 end
  if pos == 3 then return 278, 167-17*2 end
  if pos == 4 then return 134, 167-17*2 end
end
--
-- function BasicMap.getPlayerInitPosition(id)
--   if id == 1 then return 70, 58 end
--   if id == 2 then return 344, 58 end
--   if id == 3 then return 344, 170 end
--   if id == 4 then return 70, 170 end
-- end

function BasicMap.getPayload()
  return Bomb.new()
end

function BasicMap.getSafeSpotForPlayer(count, p)
  if count > map.numberOfPlayers then return nil end -- let map decide

  if map.numberOfPlayers == 2 and p.id == 2 then
    return 344, 170
  end

  if map.numberOfPlayers == 3 and p.id == 3 then
    if p.id == 3 then return 205, 170 end
  end

  if p.id == 1 then return 70, 58 end
  if p.id == 2 then return 344, 58 end
  if p.id == 3 then return 344, 170 end
  if p.id == 4 then return 70, 170 end
end

function BasicMap.onBoxRemoved(map)
  later.after(0.1, function()
    local bx, by = map.getSafeSpot()
    game.add(game.spawn('Box', bx+3, by))
  end)
end

function _isEmpty(x, y)
  if true then return false end
  if (x == 10 or x == 11) or (y==5 or y==6) then return true end
  return false
end

function BasicMap.build(_game, _map)
  game = _game
  map = _map
  local abs = math.abs
  local rand = love.math.random

  local ox, oy = 35, 20
  for x=1,20 do
    for y=1,10 do
      if not _isEmpty(x, y) then
        -- local delay = (x+y)/10
        -- local delay = ((x-10)*(y-5) + 15)/20
        -- local delay = x/40 + y/5
        -- local delay = math.abs(10 - x)/10
        local delay = 2-(abs(x-10) + abs(y-5)) / 10 + (rand() * 0.8 -0.4) -- extract?

        local tq_pos = love.math.random(1,30)
        if tq_pos > 8 then tq_pos = 1 end
        local t = Tile.new(tq_pos, ox+x*16, oy+y*16, delay, _getTileColor(x, y))
        game.add(t)
      end
    end
  end
end

function BasicMap.onClose()

end

return BasicMap
