local serialize = require('lib.serialize')

local options = {
  audio = {
    master = 100,
    music = 100,
    sound = 100,
  },
  video = {
    scale = 3,
    vsync = true,
  },
  controls = {
    kb = { up='up', left='left', down='down', right='right', jump='x', throw='c', menu='escape' },
  }
}


function options.save()
  love.filesystem.write('options', 'return ' .. serialize({
    audio = options.audio,
    video = options.video,
    controls = options.controls
  }))
end

function options.load()
  local content = love.filesystem.read('options')
  if not content then return end

  local ok, err = pcall(function()

    local chunk, err = load(content, nil, 'bt', {})
    if not chunk then
      io.stderr:write('CANNOT LOAD OPTION: ', err, '\n')
      return
    end
    local ok, t = pcall(chunk)
    if not ok then io.stderr:write('OPTIONS INVALID', '\n'); return end

    for k, v in pairs(t.audio) do
      if options.audio[k] then options.audio[k] = t.audio[k] or options.audio[k] end
    end

    for k, v in pairs(t.video) do
      if options.video[k] then options.video[k] = t.video[k] or options.video[k] end
    end

    for k, v in pairs(t.controls.kb) do
      if options.controls.kb[k] then
        options.controls.kb[k] = t.controls.kb[k] or options.controls.kb[k]
      end
    end

  end)
  if not ok then
    io.stderr:write('ERROR WHILE READING OPTIONS', err, '\n')
  end

end

function options.changeAudio(type)
  options.audio[type] = options.audio[type] + 10
  if options.audio[type] > 100 then options.audio[type] = 0 end
  options.save()
end

function options.changeVideoScale()
  if options.video.scale == 'full' then
    options.video.scale = 1
  elseif options.video.scale == 5 then
    options.video.scale = 'full'
  else
    options.video.scale = options.video.scale + 1
  end
  options.save()
end

function options.getVolume(type)
  return (options.audio.master / 100) * (options.audio[type] / 100)
end

function options.changeVsync()
  local w, h, flags = love.window.getMode()
  if flags.vsync then
    flags.vsync = false
    options.video.vsync = false
    love.window.setMode(w, h, flags)
  else
    flags.vsync = true
    options.video.vsync = true
    love.window.setMode(w, h, flags)
  end
  options.save()
end

return options
