local colors = require('utils.colors')
local later = require('utils.later')
local audio = require('utils.audio')
local map = require('map')
local input = require('input')
local game = nil
local pstates = nil
local t = 0
local join_state = {}
local nhts = {}
local goto_play = false


function join_state.setup(_game)
  game = _game
end

function join_state.init()
  input.clearAssignments()
  t = 0
  join_state.t = 0
  pstates = {}
  nhts = {}
  goto_play = false
end

--
local function pprint(id)
  if id > 4 then return end
  if goto_play and math.floor(t*8) % 2 == 0 then return end

  local x, y = 80, 30 + id * 30
  local c = colors.idToName(id):upper()

  if not pstates[id] then

    if not goto_play then
      love.graphics.setColor(colors.gray)
      love.graphics.print(('%6s: join?'):format(c), x, y)
    end

  else
    love.graphics.setColor(colors.idToColor(id))

    if #pstates == 1 then
      love.graphics.print(('%6s: waiting for another player'):format(c), x, y)
    else

      love.graphics.print(('%6s: hold jump: %s'):format(c, ('*'):rep(nhts[id])), x, y)
    end
  end
end


function join_state.draw()
  for i=1,4 do pprint(i) end
end

local function join(device)
  if #pstates >= 4 then return end

  for i=1,#pstates do
    if pstates[i] and pstates[i].device == device then return end
  end

  local id = #pstates+1

  print('ASSIGN', id, device)
  input.assign(id, device)
  pstates[id] = { device = device }
  nhts[id] = 0
  audio.playRandom('menu_confirm')
end

local function checkAllHolds()
  if #nhts <= 1 then return end

  local count = 0
  for id=1,#nhts do
    if nhts[id] >= 10 then count = count + 1 end
  end

  if count == #nhts then
    for id=1,#nhts do nhts[id] = 11 end

    goto_play = true
    audio.playRandom('menu_confirm')
    audio.fadeMusic('menu_music')
    later.after(1.5, function()
      map.numberOfPlayers = #nhts
      game.setState('play')
    end)
  end
end

function join_state.getCount()
  return #nhts
end

function join_state.update(dt)
  t = t + dt
  join_state.t = t

  if not goto_play then
    for id=1,#nhts do
      local ht = math.max(input.getHoldT(id, 'jump'), input.getHoldT(id, 'throw'))
      local T = 0.33
      nhts[id] = math.floor(math.min(1, ht / T) * 10) + 1
    end

    for device, v in input.iterAssignRequests() do
      join(device)
    end

    checkAllHolds()

  end
end




return join_state
