local signal = require('lib.signal')
local input = require('input')
local colors = require('utils.colors')
local audio = require('utils.audio')
local assets = require('utils.assets')
local later = require('utils.later')
local menu = require('utils.menu')
local options = require('options')

local FADING_TIME = 1

local game = nil
local fading_t = nil
local main_menu = nil
local menu_state = {}
local t = 0

function menu_state.setup(_game)
  game = _game
end

local function triggerPlay()

end

function menu_state.init()
  curr = 1
  t = 0
  fading_t = nil
  audio.playMusic('menu_music')
  game.unloadMap()

  -- todo: options menu should probably be in the play menu as well!
  main_menu = menu.new({
    { text = 'PLAY', cb=function()
      main_menu:finalConfirm()
      later.after(1, function()
        fading_t = FADING_TIME;
      end)
    end},
    { text = 'OPTIONS',
      submenu = {
        { text = 'CONTROLS', submenu = { title = 'controls',
          { text = 'KEYBOARD', submenu = { { text = 'KB_SHOW' } } },
          { text = 'GAMEPAD', submenu = { { text='GAMEPAD_SHOW' } } },
          { text = 'back', exit = true }
        }},
        { text = 'VIDEO', submenu = { title = 'video',
          { text = 'FULLSCREEN',
            rendertext=function()
              return ('FULLSCREEN: %s'):format(love.window.getFullscreen() and 'ON' or 'OFF')
            end,
            cb=function() love.window.setFullscreen(not love.window.getFullscreen()) end },
          { text = 'SCALE',
            rendertext=function()
              if options.video.scale == 'full' then
                return 'SCALE: FULL'
              else
                return ('SCALE: ×%d'):format(options.video.scale)
              end
            end,
            cb=function() options.changeVideoScale(); audio.play('menu_sound') end,
          },
          { text = 'VSYNC',
            rendertext=function()
              local w, h, mode = love.window.getMode()
              return ('VSYNC: %s'):format(mode.vsync and 'ON' or 'OFF')
            end,
            cb=function()
              options.changeVsync()
            end
          },
          { text = 'back', exit = true }
        }},
        { text = 'AUDIO', submenu = { title = 'audio',
          { text = 'MASTER',
            rendertext=function() return ('MASTER: %d%%'):format(options.audio.master) end,
            cb=function()
              options.changeAudio('master')
              audio.play('menu_sound')
              audio.updateMusicVolume()
            end,
          },
          { text = 'MUSIC',
            rendertext=function() return ('MUSIC: %d%%'):format(options.audio.music) end,
            cb=function()
              options.changeAudio('music')
              audio.play('menu_sound')
              audio.updateMusicVolume()
            end,
          },
          { text = 'SOUND',
            rendertext=function() return ('SOUND: %d%%'):format(options.audio.sound) end,
            cb=function() options.changeAudio('sound'); audio.play('menu_sound') end,
          },
          { text = 'back', exit = true }
        }},
        { text = 'back', exit = true }
      }
    },
    { text = 'QUIT', cb=function()
      main_menu:finalConfirm()
      audio.fadeMusic('menu_music')
      later.after(0.8, function() -- let the sound play
        love.event.quit()
      end)
    end},
    default = 'PLAY'
  })
  main_menu:enable()
end

local function _drawImgInTheMiddle(img_name)
  local img = assets.img(img_name)
  local img_w, img_h = img:getDimensions()
  local w, h = game.getDimensions()

  love.graphics.draw(img, w/2, h/2, 0, 1, 1, img_w/2, img_h/2)
end

function menu_state.hudDraw()
  if main_menu.state == 'GAMEPAD_SHOW' then
    _drawImgInTheMiddle('pad.png')

  elseif main_menu.state == 'KB_SHOW' then
    _drawImgInTheMiddle('kb.png')

  else
    if main_menu.state == 'SCALE' or main_menu.state == 'FULLSCREEN' then
      love.graphics.setColor(255, 255, 255, 200)
      local w, h = game.getDimensions()
      love.graphics.rectangle('line', 0.5+5, 0.5+5, w-10, h-10)
    end

    main_menu:draw()
  end
end

function menu_state.draw()
end

function menu_state.update(dt)
  t = t + dt

  if main_menu.state == 'GAMEPAD_SHOW' and input.anyHit(1) then
    main_menu:exitOneLevel()
    main_menu:set('GAMEPAD')
  else
    main_menu:update(dt)
  end


  if fading_t then
    fading_t = fading_t - dt
    if fading_t < 0 then game.setState('tutorial') end
  end
end


return menu_state
