local points = require('utils.points')
local colors = require('utils.colors')
local animset = require('utils.animset')
local assets = require('utils.assets')
local audio = require('utils.audio')
local later = require('utils.later')
local input = require('input')
local player_dust = require('particles.player_dust')
local drawFade = require('utils.fade')


local game = nil
local player_animset = nil
local newAnimset = nil
local t = 0
local p_prev, p_target, p_delays, p_confirms = nil, nil, nil, nil
local is_fading = false
local fade = 0

local show_points_state = {}
local dance_dir = 1


function show_points_state.setup(_game)
  game = _game
  newAnimset = animset.define({
    default =  { 'player_idle',  0.10 },
    sad = { 'player_sad', 0.10 }
  })
  player_animset = newAnimset('default')
  psystem = player_dust.new()
  psystem:setColors(255, 255, 255, 200, 255, 255, 255, 150)
  psystem:setBufferSize(200)
end

function show_points_state.init()
  t = 0
  p_prev = { points.getPrev() }
  p_target = { points.getValues() }
  p_delays = {0.5,1.0,1.5,2.0}
  p_done = {false, false, false, false}
  p_confirms = { false, false, false, false }
  psystem:reset()
  is_fading = false
  fade = 0
end

local f = 0
function show_points_state.draw()
  f = f + 1
  local best_score = math.max(points.getValues())
  local scale = 1 / (math.floor(best_score/100) + 1)

  if f%20 == 0 then
    dance_dir = -dance_dir
  end
  local number_of_players = game.map.numberOfPlayers

  for i=1,number_of_players do
    -- todo number of players

    local min = p_prev[i]
    local max = p_target[i]
    local actual_score = max

    local val = min + (max-min) * math.min(math.max(0, t-p_delays[i]), 1)^3

    local h = val*2 * scale
    local w = 60
    local x = i*70+16
    local y = 240-h

    if number_of_players == 2 then
      x = x + 60
    elseif number_of_players == 3 then
      x = x + 30
    end

    if (val == max) and (not p_done[i]) then
      p_done[i] = true
      audio.playRandom('explosion')
      psystem:moveTo(x+15, y)
      psystem:emit(16)
      psystem:moveTo(x+40, y)
      psystem:emit(16)
    end

    love.graphics.setColor(colors.idToColor(i))
    love.graphics.rectangle('fill', x, y, w, h)
    love.graphics.setColor(255, 255, 255, 128)
    love.graphics.rectangle('fill', x, y, w, 1)
    love.graphics.setColor(colors.idToColor(i))

    local dir = i > 2 and -1 or 1

    local blink = p_confirms[i] and ((math.floor(t*4))%2==0)
    if blink then game.toggleBlink(true) end
    player_animset:draw(x+15, y-32, dance_dir, 0, 2, 2)
    if blink then game.toggleBlink(false) end
    love.graphics.printf(math.floor(val), x, y-34, w, 'center')
  end

  love.graphics.setColor(colors.white)
  love.graphics.draw(psystem)

  if fade > 0 then
    love.graphics.setColor(0,0,0,255*fade)
    love.graphics.rectangle('fill', -10000, -10000, 20000, 20000)
  end
end

function show_points_state.hudDraw()
  if t < 1 then
    drawFade(1-t/1)
  end
end

function show_points_state.update(dt)
  player_animset:update(dt)
  t = t + dt
  psystem:update(dt)

  -- todo handle less players
  local number_of_players = game.map.numberOfPlayers
  if t > 1 then
    local count = 0
    for i=1,4 do
      if input.isConfirmHit(i) and p_confirms[i] ~= true then
        p_confirms[i] = true
        audio.playRandom('menu_confirm')
      end
      if p_confirms[i] == true then count = count + 1 end
    end

    if not is_fading and (count == number_of_players or (count >= number_of_players)) then
      is_fading = true
      later.after(0.5, function()
        audio.play('next_stage')
        later.xafter(32, 1.8/32, function()
          fade = fade + 1/32
        end)
      end)
      audio.fadeMusic('play_music') -- fade takes 1.5s (must be less then the after below)
      later.after(2.5, function()
        game.setState('play')
      end)
    end

  end
end

return show_points_state
