local anim8 = require('lib.anim8')
local assets = require('utils.assets')
local tex = assets.img('tex.png')
local animset = {}

local mt = {}
mt.__index = mt

function mt:draw(x, y, dir, r, sx, sy)
  self.curr_anim.flippedH = (dir == -1)
  self.curr_anim:draw(tex, x, y, r, sx, sy)
end

function mt:update(dt)
  self.curr_anim:update(dt)
  self.t = self.t + dt
end

function mt:getTime()
  return self.t
end

function mt:set(state_name, restart)
  self.curr_state_name = state_name
  self.curr_anim = self.anims[state_name]
  if restart then self.curr_anim:gotoFrame(1) end
  self.t = 0
end

function mt:substituteDraw(substitute_name, x, y, dir)
  local sub = self.anims[substitute_name]
  print(">", sub)
  local curr = self.curr_anim
  sub.position, sub.timer = curr.position, curr.timer

  self.curr_anim = sub
  self:draw(x, y, dir)
  self.curr_anim = curr
end

function mt:getFrame()
  return self.curr_anim.position
end

function mt:getState()
  return self.curr_state_name
end

function mt:play(state_name)
  if self:getState() ~= state_name then
    self:set(state_name, true)
    self.t = 0
  end
end


function animset.define(def)
  return function(init_state)
    local a = setmetatable({
      anims = {},
      t = 0
    }, mt)

    for k, v in pairs(def) do
      a.anims[k] = anim8.newAnimation(assets.quadsGroup(v[1]), v[2] or 0.1)
    end

    a:set(init_state)

    return a
  end
end

return animset
