local later = require('utils.later')
local options = require('options')

local audio = {}

local _sources = {}
local _music = {}

local function _findReadySource(name)
  local sources = _sources[name].sources
  for i, v in ipairs(sources) do
    if not v:isPlaying() then return v end
  end
end

function audio.playRandom(name, vary)
  vary = vary or 0.4
  local pitch_val = 1 + (love.math.random()*vary) - (vary/2)

  audio.play(name, pitch_val)
end

function audio.play(name, pitch_val)
  local s = _findReadySource(name)
  if s then
    s:setPitch(pitch_val or 1)
    s:setVolume(options.getVolume('sound'))
    s:play()
  -- else we loose a sound - not a big deal
  end
end

function newAudio(name, count, volume)
  local a = {
    name = name,
    sources = {},
    count = count
  }
  local sound = love.audio.newSource('assets/audio/'..name..'.wav', 'static')
  a.sources[1] = sound
  for i=2,count do
    a.sources[i] = sound:clone()
  end
  _sources[name] = a
end

local DEFAULT_MUSIC_VOLUME = 0.5 -- tmp about 0.2

function newMusic(name)
  local m = love.audio.newSource('assets/audio/'..name..'.ogg')
  _music[name] = m
  m:setLooping(true)
  m:setVolume(DEFAULT_MUSIC_VOLUME)
end

function audio.fadeMusic(name)
  local init_vol = 1
  local vol = init_vol
  local steps = 8
  local time = 1.5
  later.xafter(steps, time/steps, function()
    vol = vol - (init_vol / steps)
    _music[name]:setVolume(vol * DEFAULT_MUSIC_VOLUME * options.getVolume('music'))
    if vol <= 0.01 then
      _music[name]:stop()
      _music[name]:setVolume(1)
    end
  end)
end
function audio.playMusic(name)
  for k, v in pairs(_music) do
    v:stop()
  end
  print(options.getVolume('music'))
  _music[name]:setVolume(DEFAULT_MUSIC_VOLUME * options.getVolume('music'))
  _music[name]:play()
end

function audio.updateMusicVolume()
  for k, v in pairs(_music) do
    v:setVolume(DEFAULT_MUSIC_VOLUME * options.getVolume('music'))
  end
end

function audio.setup()
  newAudio('explosion', 12)
  newAudio('bomb_jump', 12)
  newAudio('jump', 5)
  newAudio('fall', 5)
  newAudio('tile_fall', 20)
  newAudio('head_fall', 8)
  newAudio('stomp', 5)
  newAudio('menu_sound', 3)
  newAudio('menu_confirm', 4)
  newAudio('new_life', 5)
  newAudio('fall_die', 5)
  newAudio('intro_fall', 5)
  newAudio('tile_up', 60)
  newAudio('pick', 10)
  newAudio('throw', 6)
  newAudio('next_stage', 1)
  newAudio('join', 4)

  newMusic('play_music')
  newMusic('menu_music')
end

return audio
