local colors = require('utils.colors')
local audio = require('utils.audio')
local input = require('input')

local menu = {}

local mt = {}
mt.__index = mt

function mt:menuPrint(curr, t, y, len)
  local w, _ = game.getDimensions()
  if curr then t = '- ' .. t .. ' -' end
  local is_red = curr and ((not self._final_confirm) or (math.floor(self.t*10)%2==0))
  love.graphics.setColor(is_red and colors.red or colors.white)
  love.graphics.printf(t, 0, 60+y*22, w, 'center', 0, 1, 1)
end

function mt:draw()
  if not self.enabled then return end

  if self.def.fade then
    love.graphics.setColor(0, 0, 0, self.def.fade * 255)
    love.graphics.rectangle('fill', 0, 0, game.getDimensions())
  end

  -- local len = #self.def
  for i, v in ipairs(self.def) do
    local t = v.rendertext and v.rendertext() or v.text
    self:menuPrint(self.state == v.text, t, i, 0)
  end

  love.graphics.setColor(colors.white)
  if self.def.title then
    local w, _ = game.getDimensions()
    love.graphics.printf(self.def.title, 0, 40, w, 'center', 0, 1, 1)
  end
end

function mt:set(name)
  self.state = name
end

function mt:disable()
  self._disabled_in_current_frame = true
  self.enabled = false
end

function mt:enable()
  if self._disabled_in_current_frame then return end -- avoid race conditions
  if self._final_confirm then return end -- avoid race conditions
  self:setDefault() -- alwyas start at the same place
  self.enabled = true
end

function mt:isEnabled()
  return self.enabled
end

function mt:getCurrentItem()
  return self.def[self:getCurrentPos()]
end

function mt:getCurrentPos()
  for i, v in ipairs(self.def) do
    if v.text == self.state then return i end
  end
end

function mt:nav(dir)
  local pos = self:getCurrentPos(); if not pos then return end
  local item = self.def[pos + dir]; if not item then return end

  self:set(item.text)
  audio.playRandom('menu_sound')
end

function mt:exitOneLevel()
  if self.def.parent_def then
    self.def = self.def.parent_def
    audio.playRandom('menu_confirm')
    self:setDefault()
  else
    audio.playRandom('menu_confirm')
    self:disable()
  end
end

function mt:finalConfirm()
  self._final_confirm = true
  audio.playRandom('menu_confirm')
  -- some blinking...
end

function mt:setDefault()
  self:set(self.def.default or self.def[1].text)
end

function mt:update(dt)
  self.t = self.t + dt
  self._disabled_in_current_frame = false

  if self._final_confirm then return end
  if not self.enabled then return end

  local up = input.isUpHit(1)
  local down = input.isDownHit(1)

  if down then self:nav(1) end
  if up then self:nav(-1) end

  if self.def.on_menu_buton and (input.isMenuHit(1) or input.isCancelHit(1)) then
    if self.def.on_menu_buton == 'exit' then
      self:exitOneLevel()
    else
      self.def.on_menu_buton()
    end
  elseif input.isConfirmHit(1) then
    local item = self:getCurrentItem()
    if item then
      if item.exit then
        self:exitOneLevel()
      elseif item.submenu then
        item.submenu.parent_def = self.def
        self.def = item.submenu
        self:setDefault()
        self.def.on_menu_buton = self.def.on_menu_buton or 'exit' -- default for submenus
        audio.playRandom('menu_confirm')
      elseif item.cb then
        item.cb()
      end
    end
  end
end

--[[

menu.new({
  { text='RESUME', exit=true },
  { text='QUIT', cb=function()end },
  default = 'RESUME',
  disable_on_menu_buton = true,
})

]]

function menu.new(def, options)
  options = options or {}
  local m = setmetatable({ def = def, full_def = def, enabled=false, fade=options.fade, t=0 }, mt)
  return m
end


return menu
